/***************************************************************************
 *   Copyright (C) 2005 by Nguyen The Toan   *
 *   nguyenthetoan@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef _KXMAME_H_
#define _KXMAME_H_

#define statusBarGameStatId	9
#define statusBarGameTotalId	10

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qlabel.h>
#include <qlineedit.h>

#include <kapplication.h>
#include <kmainwindow.h>
#include <kactionclasses.h>
#include <kpopupmenu.h>

#include "kxmameview.h"
#include "koselectaction.h"

/**
 * This class serves as the main window for kxmame.  It handles the
 * menus, toolbars, and status bars.
 *
 * @short Main window class
 * @author Nguyen The Toan <nguyenthetoan@gmail.com>
 * @version 1.0.2
 */
class kxmame : public KMainWindow
{
    	Q_OBJECT
public:
	kxmame();
	virtual ~kxmame();
	
protected:
	/**
	* This function is called when it is time for the app to save its
	* properties for session management purposes.
	*/
	void saveProperties(KConfig *);
	
	/**
	* This function is called when this app is restored.  The KConfig
	* object points to the session management config file that was saved
	* with @ref saveProperties
	*/
	void readProperties(KConfig *);
	

private slots:

	void exit();
	void romOptionsPreferences();
	void optionsPreferences();
	void directoriesSetup();
	void setupExecutablesSelection();
	void set_current_executable (XmameExecutable *new_exec);
	void execChanged();
	void gameList();
	void gameRefresh();
	void gameAudit();
	void cloneColor();
	void toggleSearch();
	
	void changeStatusbar(RomEntry *rom);
	void changeCaption(const QString& text);
	void changeExpandCollapse();
	void changeFavorite(bool);

private:
	void setupAccel();
	void setupActions();

	bool showSearch;
	kxmameView *m_view;
	KAction *mConfigAction,*mAuditAllAction, *mExpandAction, *mCollapseAction;
	KAction *mRefreshAction;
	KoSelectAction *mExecutablesAction;
	KAction *mAddFavoriteAction;
	KAction *mRemoveFavoriteAction;
};

#endif // _KXMAME_H_

