/***************************************************************************
 *   Copyright (C) 2005 by Nguyen The Toan   *
 *   nguyenthetoan@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef _KXMAMEVIEW_H_
#define _KXMAMEVIEW_H_

#include <qwidget.h>
#include <qlabel.h>
#include <qtextedit.h>
#include <qpoint.h>
#include <qlayout.h>
#include <qsplitter.h>
#include <klistbox.h>
#include <klistview.h>
#include <kpopupmenu.h>
#include <kapplication.h>
#include <kmainwindow.h>
#include <ksplashscreen.h>
#include <klistviewsearchline.h>

#include "ColorListViewItem.h"
#include "kxmamesnaptab.h"
#include "kxmameSplash.h"

extern "C" {
#include "game_list.h"
#include "unzip.h"
}

class QPainter;

/**
 * This is the main view class for kxmame.  Most of the non-menu,
 * non-toolbar, and non-statusbar (e.g., non frame) GUI code should go
 * here.
 *
 * This kxmame uses an HTML component as an example.
 *
 * @short Main view
 * @author Nguyen The Toan <nguyenthetoan@gmail.com>
 * @version 0.1
 */
class kxmameView : public QWidget
{
	Q_OBJECT
public:

	kxmameView(QWidget *parent);
	virtual ~kxmameView();

	void finalize();
	
	void changeGameListView(ListMode newMode);
	void updateFilterList();
	void filterGames();
	
	void updateGameListView();
	
	void setupGameIcons();
	void saveGameColumnWidth();
	void saveGameColumnSort();
	void restoreGameColumnWidth();
	
	void setScaledScreenshot() {mScreenTab->setScaledScreenshot();}
	
	KPopupMenu *gameListContextMenu;
	QSplitter* splitter4;
	QSplitter* splitter3;
	KListView* gameListView;
	KToolBar *searchToolBar;
        //KListViewSearchLine* search;
        //QLabel* searchLabel;

signals:
	
	void signalChangeStatusbar(RomEntry *tmprom);
	void signalChangeCaption(const QString& text);
	
	void signalChangeExpandCollapse();
	void signalChangeFavorite(bool);

public slots:
	//void iGameSearch(const QString &);
	
private slots:

	void playGame();
	void playRecordGame();
	void playbackGame();
	//void playRandomGame();
	void addFavorites();
	void removeFavorites();
	void listView();
	void listTreeView();
	void detailView();
	void detailTreeView();
	void collapseGameList();
	void expandGameList();
	void columnsLayout();
	
	void toggleFilter();
	void toggleSide();
	
	void slotFolderFilterChanged(QListViewItem *);
	void slotGameListSelectionChanged();
	
	void gameListContext(KListView *, QListViewItem *, const QPoint &);

protected:
	void tree(bool);
	void sortGameList();
	bool set_history (const QString entry_name, QString *text_buffer);
	bool set_info (const QString entry_name, QString *text_buffer);
		
	int gameNameColumnId;
	KMainWindow *mParent;
	QHBoxLayout *top_layout;
	KListView* gameFilterList;
	QTextEdit* gameInfoView;
	kxmameSnapTab *mScreenTab;
	
	QPixmap get_icon_for_rom (RomEntry *rom, guint size, ZIP *zip);

};

extern KListView *mGameListView;

const QString column_title (int column_num);


#endif // _KXMAMEVIEW_H_
