/***************************************************************************
 *   Copyright (C) 2005 by Nguyen The Toan   *
 *   nguyenthetoan@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <kdialog.h>
#include <klocale.h>
#include <kdebug.h>

#include "screentab.h"
//#include "kxinewidget.h"

#include <klibloader.h>
#include <kparts/factory.h>
#include <kparts/part.h>

class QTime;

#include <qvariant.h>
#include <qpushbutton.h>
#include <qtabwidget.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qtimer.h>


/*
 *  Constructs a MySnapTab as a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'.
 */
MySnapTab::MySnapTab( QWidget* parent, const char* name, WFlags fl )
	: QWidget( parent, name, fl )
{
	if ( !name )
		setName( "MySnapTab" );
	setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)7, 0, 0, sizePolicy().hasHeightForWidth() ) );
	MySnapTabLayout = new QHBoxLayout( this, 0, 6, "MySnapTabLayout"); 

	screenshotTab = new QTabWidget( this, "screenshotTab" );
	screenshotTab->setTabShape( QTabWidget::Triangular );
	screenshotTab->setMargin( 0 );

	tab = new QWidget( screenshotTab, "tab" );
	tabLayout = new QHBoxLayout( tab, 0, 6, "tabLayout"); 

	snapPixmap = new kxmameSnapLabel( tab, "snapPixmap" );
	snapPixmap->setMinimumSize( QSize( 320, 240 ) );
	tabLayout->addWidget( snapPixmap );
	screenshotTab->insertTab( tab, QString::fromLatin1("") );

	tab_2 = new QWidget( screenshotTab, "tab_2" );
	tabLayout_2 = new QHBoxLayout( tab_2, 0, 6, "tabLayout_2"); 

	flyerPixmap = new kxmameSnapLabel( tab_2, "flyerPixmap" );
	flyerPixmap->setMinimumSize( QSize( 320, 240 ) );
	tabLayout_2->addWidget( flyerPixmap );
	screenshotTab->insertTab( tab_2, QString::fromLatin1("") );

	TabPage = new QWidget( screenshotTab, "TabPage" );
	TabPageLayout = new QHBoxLayout( TabPage, 0, 6, "TabPageLayout"); 

	cabPixmap = new kxmameSnapLabel( TabPage, "cabPixmap" );
	cabPixmap->setMinimumSize( QSize( 320, 240 ) );
	TabPageLayout->addWidget( cabPixmap );
	screenshotTab->insertTab( TabPage, QString::fromLatin1("") );

	TabPage_2 = new QWidget( screenshotTab, "TabPage_2" );
	TabPageLayout_2 = new QHBoxLayout( TabPage_2, 0, 6, "TabPageLayout_2"); 

	marqueePixmap = new kxmameSnapLabel( TabPage_2, "marqueePixmap" );
	marqueePixmap->setMinimumSize( QSize( 320, 240 ) );
	TabPageLayout_2->addWidget( marqueePixmap );
	
	screenshotTab->insertTab( TabPage_2, QString::fromLatin1("") );

	TabPage_3 = new QWidget( screenshotTab, "TabPage_3" );
	TabPageLayout_3 = new QHBoxLayout( TabPage_3, 0, 6, "TabPageLayout_3"); 

	titlePixmap = new kxmameSnapLabel( TabPage_3, "titlePixmap" );
	titlePixmap->setMinimumSize( QSize( 320, 240 ) );
	TabPageLayout_3->addWidget( titlePixmap );
	screenshotTab->insertTab( TabPage_3, QString::fromLatin1("") );
	
#ifdef ENABLE_VIDEO	
	TabPage_4 = new QWidget( screenshotTab, "TabPage_4" );
	TabPageLayout_4 = new QVBoxLayout( TabPage_4, 0, 6, "TabPageLayout_4"); 

	noVideoLabel = new kxmameSnapLabel( TabPage_4 );
	noVideoLabel->setMinimumSize( QSize( 320, 240 ) );
	TabPageLayout_4->addWidget( noVideoLabel );
	
	// Try to find libkaffeinepart
	KLibFactory *factory = KLibLoader::self()->factory( "libkaffeinepart" );
	if (factory) {
		// Create the part
		mVideoPreview = (KParts::ReadOnlyPart *)factory->create( TabPage_4, "KaffeinePart",
                	"KParts::ReadOnlyPart" );
		mVideoPreview->widget()->setMinimumSize( QSize( 320, 240 ) );
		TabPageLayout_4->addWidget( mVideoPreview->widget() );
	} else {
		mVideoPreview = NULL;
		kdDebug() << i18n("No libkaffeinepart found! Please install kaffeine if you want video preview.")
			  << endl;
	}

	screenshotTab->insertTab( TabPage_4, QString::fromLatin1("") );
	if(!mVideoPreview) {
		noVideoLabel->setOnOff(TRUE);
	} else {
		noVideoLabel->setOnOff(FALSE);
	}

#endif
	MySnapTabLayout->addWidget( screenshotTab );
	languageChange();
	resize( QSize(644, 407).expandedTo(minimumSizeHint()) );
	clearWState( WState_Polished );
}

/*
 *  Destroys the object and frees any allocated resources
 */
MySnapTab::~MySnapTab()
{
#ifdef ENABLE_VIDEO
	if (mVideoPreview) delete mVideoPreview;
#endif
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void MySnapTab::languageChange()
{
	screenshotTab->changeTab( tab, tr2i18n( "S&nap" ) );
	screenshotTab->changeTab( tab_2, tr2i18n( "Fl&yer" ) );
	screenshotTab->changeTab( TabPage, tr2i18n( "C&ab" ) );
	screenshotTab->changeTab( TabPage_2, tr2i18n( "Mar&quee" ) );
	screenshotTab->changeTab( TabPage_3, tr2i18n( "Title" ) );
#ifdef ENABLE_VIDEO
	screenshotTab->changeTab( TabPage_4, tr2i18n( "Video" ) );
#endif
}

#include "screentab.moc"
